package io.tswf.groovy.bettergroovy.asm.cache

import org.objectweb.asm.ClassReader
import org.objectweb.asm.tree.ClassNode

import java.util.concurrent.ConcurrentHashMap
import java.util.function.Function

@Singleton(lazy = true)
class ClassNodeCache {

    Function<String, InputStream> resourceReader = {Thread.currentThread().contextClassLoader.getResourceAsStream(it.toString())}
    Map<String, ClassNode> classNodes = new ConcurrentHashMap<>()

    ClassNode getClassNode(String name) {
        if (name == null) {
            return null
        }

        if (name.startsWith("L")) {
            name = name.substring(1, name.length() - 1)
        }

        if (name.contains(".")) {
            name = name.replace(".", "/")
        }

        if (!name.endsWith(".class")) {
            name += ".class"
        }

        return classNodes.computeIfAbsent(name, {
            return getClassNodeFromStream(resourceReader.apply(name), false)
        })
    }

    ClassNode getClassNodeFromStream(InputStream is, boolean putCache = true) {
        if (is == null) {
            return null
        }

        def cr = new ClassReader(is)
        def cn = new ClassNode()
        cr.accept(cn, ClassReader.SKIP_FRAMES)

        if (putCache) {
            if (cn.javaName !in classNodes.keySet()) {
                classNodes[cn.javaName] = cn
            }
        }

        return cn
    }
}
