package io.tswf.groovy.bettergroovy.asm.defaults

import com.google.auto.service.AutoService
import groovy.transform.CompileStatic
import io.tswf.groovy.bettergroovy.asm.extension.ITypeDefaultValueProvider
import org.objectweb.asm.MethodVisitor
import org.objectweb.asm.Type

import static org.objectweb.asm.Opcodes.*

@AutoService(ITypeDefaultValueProvider)
@CompileStatic
class PrimitiveIConstDefaultValueProvider implements ITypeDefaultValueProvider {
    Collection types = [
            Type.BYTE_TYPE,
            Type.SHORT_TYPE,
            Type.INT_TYPE,
            Type.BOOLEAN_TYPE,
            Type.CHAR_TYPE
    ]

    @Override
    void pushValue(MethodVisitor methodVisitor) {
        methodVisitor.visitInsn(ICONST_0)
    }
}

@AutoService(ITypeDefaultValueProvider)
@CompileStatic
class PrimitiveFloatDefaultValueProvider implements ITypeDefaultValueProvider {
    Collection types = [
            Type.FLOAT_TYPE
    ]

    @Override
    void pushValue(MethodVisitor methodVisitor) {
        methodVisitor.visitInsn(FCONST_0)
    }
}

@AutoService(ITypeDefaultValueProvider)
@CompileStatic
class PrimitiveDoubleDefaultValueProvider implements ITypeDefaultValueProvider {
    Collection types = [
            Type.DOUBLE_TYPE
    ]

    @Override
    void pushValue(MethodVisitor methodVisitor) {
        methodVisitor.visitInsn(DCONST_0)
    }
}

@AutoService(ITypeDefaultValueProvider)
@CompileStatic
class StringDefaultValueProvider implements ITypeDefaultValueProvider {
    Collection types = [
            Type.getType(String)
    ]

    @Override
    void pushValue(MethodVisitor methodVisitor) {
        methodVisitor.visitLdcInsn("")
    }
}