package io.tswf.groovy.bettergroovy.asm.member.filters.standard

import com.google.auto.service.AutoService
import groovy.transform.CompileStatic
import io.tswf.groovy.bettergroovy.asm.member.IMember
import io.tswf.groovy.bettergroovy.asm.member.filters.IGeneratedMemberFilter
import org.objectweb.asm.Type

@AutoService(IGeneratedMemberFilter)
@CompileStatic
class GeneratedAnnotationMemberFilter implements IGeneratedMemberFilter {

    List<Type> generatedAnnotations = ServiceLoader.load(IGeneratedAnnotationsProvider)
        .toList()
            .stream()
                .map { it.getGeneratedAnnotationTypes()?.stream() }
                .filter { it != null }
                .flatMap { it }
                .toList()

    @Override
    boolean isGenerated(IMember member) {
        return generatedAnnotations.any { member.hasAnnotation(it) }
    }

    static interface IGeneratedAnnotationsProvider {
        Set<Type> getGeneratedAnnotationTypes()
    }

    @AutoService(IGeneratedAnnotationsProvider)
    static class JavaxGeneratedAnnotationProvider implements IGeneratedAnnotationsProvider {

        @Override
        Set<Type> getGeneratedAnnotationTypes() {
            return [ Type.getObjectType("javax/annotation/processing/Generated") ] as Set
        }
    }

    @AutoService(IGeneratedAnnotationsProvider)
    static class LombokGeneratedAnnotationProvider implements IGeneratedAnnotationsProvider {

        @Override
        Set<Type> getGeneratedAnnotationTypes() {
            return [ Type.getObjectType("lombok/Generated") ] as Set
        }
    }

    @AutoService(IGeneratedAnnotationsProvider)
    static class GroovyGeneratedAnnotationProvider implements IGeneratedAnnotationsProvider {

        @Override
        Set<Type> getGeneratedAnnotationTypes() {
            return [ Type.getObjectType("groovy/transform/Generated") ] as Set
        }
    }
}
