package io.tswf.groovy.bettergroovy.asm.member.filters.standard

import com.google.auto.service.AutoService
import groovy.transform.CompileStatic
import io.tswf.groovy.bettergroovy.asm.member.IMember
import io.tswf.groovy.bettergroovy.asm.member.filters.IGroovyInternalMemberFilter
import org.objectweb.asm.Type

@AutoService(IGroovyInternalMemberFilter)
@CompileStatic
class GroovyInternalMemberFilter implements IGroovyInternalMemberFilter {

    public static final Type GROOVY_OBJECT_TYPE = GroovyObject.asAsmType()
    public static final Type GROOVY_GENERATED_ANNOTATION_TYPE = Type.getType("groovy/transform/Generated")
    public static final Type GROOVY_INTERNAL_ANNOTATION_TYPE = Type.getType("groovy/transform/Internal")

    @Override
    boolean isGroovyInternal(IMember member) {
        if (member.hasAnnotation(GROOVY_GENERATED_ANNOTATION_TYPE)) {
            return true
        }

        if (member.hasAnnotation(GROOVY_INTERNAL_ANNOTATION_TYPE)) {
            return true
        }

        if (member.type == GROOVY_OBJECT_TYPE) {
            return true
        }

        return false
    }
}
