package io.tswf.groovy.bettergroovy.asm.member.filters.standard

import com.google.auto.service.AutoService
import groovy.transform.CompileStatic
import io.tswf.groovy.bettergroovy.asm.member.IMember
import io.tswf.groovy.bettergroovy.asm.member.filters.ITransientMemberFilter
import org.objectweb.asm.Type

import java.lang.reflect.Modifier

@AutoService(ITransientMemberFilter)
@CompileStatic
class TransientAnnotationsMemberFilter implements ITransientMemberFilter {

    List<Type> transientAnnotations = ServiceLoader.load(ITransientAnnotationTypesProvider)
        .toList()
            .stream()
                .map { it.getTransientAnnotationTypes()?.stream() }
                .filter { it != null }
                .flatMap { it }
                .toList()

    @Override
    boolean isTransient(IMember member) {
        return transientAnnotations.any { member.hasAnnotation(it) }
    }

    static interface ITransientAnnotationTypesProvider {
        Set<Type> getTransientAnnotationTypes()
    }

    @AutoService(ITransientAnnotationTypesProvider)
    static class JavaBeansTransientAnnotationProvider implements ITransientAnnotationTypesProvider {

        @Override
        Set<Type> getTransientAnnotationTypes() {
            return [ Type.getObjectType("java/beans/Transient") ] as Set
        }
    }

    @AutoService(ITransientAnnotationTypesProvider)
    static class KotlinTransientAnnotationProvider implements ITransientAnnotationTypesProvider {

        @Override
        Set<Type> getTransientAnnotationTypes() {
            return [ Type.getObjectType("kotlin/jvm/Transient") ] as Set
        }
    }
}
