package io.tswf.groovy.bettergroovy.asm.util

import groovy.transform.CompileStatic
import org.objectweb.asm.Type
import org.objectweb.asm.tree.AnnotationNode

@Singleton(lazy = true)
@CompileStatic
class AnnotationSearchUtil {

    boolean hasAnnotation(List<AnnotationNode> annotations, Class annotationClass) {
        return hasAnnotation(annotations, Type.getType(annotationClass))
    }

    boolean hasAnnotation(List<AnnotationNode> annotations, Type annotationClassType) {
        return annotations.any { it.desc == annotationClassType.descriptor }
    }

    AnnotationNode getAnnotationByType(List<AnnotationNode> annotations, Class type, String ownerName) {
        return getAnnotationByType(annotations, Type.getType(type), ownerName)
    }

    AnnotationNode getAnnotationByType(List<AnnotationNode> annotations, Type type, String ownerName) {
        return findAnnotationByType(annotations, type)
                .ensureNotNull { new NoSuchElementException("Annotation with type '${type.descriptor}' does not presented in ${ownerName}!") }
    }

    AnnotationNode findAnnotationByType(List<AnnotationNode> annotations, Class type) {
        return findAnnotationByType(annotations, Type.getType(type))
    }

    AnnotationNode findAnnotationByType(List<AnnotationNode> annotations, Type type) {
        return annotations.find { it.desc == type.descriptor }
    }
}
