package io.tswf.groovy.bettergroovy.asm.util

import io.tswf.groovy.bettergroovy.asm.cache.ClassNodeCache
import org.objectweb.asm.tree.AnnotationNode
import org.objectweb.asm.tree.ClassNode

import java.util.function.Predicate

@Singleton(lazy = true)
class ClassNodeUtil {



    boolean hasAnnotationDeep(ClassNode cn, Predicate<AnnotationNode> filter) {
        return hasAnnotationDeepInternal(cn, filter, new HashSet<String>())
    }

    protected boolean hasAnnotationDeepInternal(
            ClassNode cn,
            Predicate<AnnotationNode> filter,
            Set<String> visitedClassNodes
    ) {
        if (cn == null) {
            return false
        }

        if (!visitedClassNodes.add(cn.name)) {
            return false
        }

        def allAnnotations = cn.allAnnotations
        def passedAnnotation = allAnnotations.find {filter.test(it)}
        if (passedAnnotation != null) {
            return true
        }

        def allAnnotationTypes = allAnnotations
                .map {it.type}
                .map { ClassNodeCache.instance.getClassNode(it.className)}

        return allAnnotationTypes.any {hasAnnotationDeepInternal(it, filter, visitedClassNodes)}
    }
}
