package io.tswf.groovy.bettergroovy.asm.util

import groovy.transform.CompileStatic
import org.objectweb.asm.Opcodes
import org.objectweb.asm.Type
import org.objectweb.asm.tree.AbstractInsnNode
import org.objectweb.asm.tree.FieldInsnNode
import org.objectweb.asm.tree.InsnNode
import org.objectweb.asm.tree.LdcInsnNode

import static org.objectweb.asm.Opcodes.ICONST_0
import static org.objectweb.asm.Opcodes.ICONST_1

@Singleton(lazy = true)
@CompileStatic
class ConstantInsnUtil {

    AbstractInsnNode getConstantInsn(Object constant) {
        if (constant == null) {
            return new InsnNode(Opcodes.ACONST_NULL)
        }

        if (constant instanceof Integer) {
            return (new LdcInsnNode((int) constant))
        } else if (constant instanceof Float) {
            return (new LdcInsnNode((float) constant))
        } else if (constant instanceof String) {
            return (new LdcInsnNode((String) constant))
        } else if (constant instanceof Type) {
            return (new LdcInsnNode((Type) constant))
        } else if (constant instanceof Class) {
            return (new LdcInsnNode(Type.getType((Class<?>) constant)))
        } else if (constant.getClass().isEnum()) {
            return (new FieldInsnNode(Opcodes.GETSTATIC, Type.getInternalName(constant.getClass()), ((Enum<?>) constant).name(), Type.getDescriptor(constant.getClass())))
        } else if (constant instanceof Boolean) {
            return new InsnNode(((Boolean)constant) ? ICONST_1 : ICONST_0)
        } else {
            // Неизвестный тип константы
            throw new IllegalArgumentException("Unsupported constant type: " + constant.getClass().getName())
        }
    }
}
