package io.tswf.groovy.bettergroovy.asm.util

import groovy.transform.CompileStatic
import org.objectweb.asm.Type
import org.objectweb.asm.tree.AnnotationNode
import org.objectweb.asm.tree.LabelNode
import org.objectweb.asm.tree.LocalVariableNode
import org.objectweb.asm.tree.MethodNode
import org.objectweb.asm.tree.ParameterNode

@Singleton(lazy = true)
@CompileStatic
class MethodNodeUtil {

    int createNewLocalVariable(MethodNode methodNode, Type type, String namePrefix = "var", boolean randomize = false) {
        def index = methodNode.maxLocals

        // Увеличиваем maxLocals на размер типа переменной
        def typeSize = type.getSize()
        methodNode.maxLocals += typeSize

        def randomPayload = randomize ? "_${Random.randomString(8)}" : ""

        // Создаем новую локальную переменную
        def localVariable = new LocalVariableNode("${namePrefix}${randomPayload}_${index}", type.getDescriptor(), null, new LabelNode(), new LabelNode(), index)

        methodNode.instructions.insert(localVariable.start)
        methodNode.instructions.add(localVariable.end)

        methodNode.localVariables.add(localVariable)

        return index
    }

    List<AnnotationNode> getAllParameterAnnotations(MethodNode methodNode, int parameterIndex) {
        def result = new ArrayList<AnnotationNode>()
        methodNode?.visibleParameterAnnotations?[parameterIndex]?.let {
            result.addAll(it)
        }

        methodNode?.invisibleParameterAnnotations?[parameterIndex]?.let {
            result.addAll(it)
        }

        return result
    }
}
